/*******************************************************************************
*
*         USING THE EXTERNAL INTERRUPT 0 TO ENTER AN INTERRUPT SUBROUTINE
*         ---------------------------------------------------------------
*
*   Filename:                                    
*   Autor:                  David Kress
*
*   Version:                1.0
*   Date:                   24. November 2010
*   Built with:             ATMEL AVR Studio 4.14 and AVR-GCC; Optimization -01
*
*   Description:            This programm is a simple example how to detect an
*                           external interrupt on INT0 and entering the
*                           interrupt subroutine. The subroutine will toggle 
*                           a digitalport which can be visualized with an led or
*                           or a dc voltmeter.The interrupt is caused by a
*                           faling edge.
*                           Notice, that this program doesn't contains 
*                           switch debouncing. So it could be, that the led goes
*                           off again immediately.
*                           
*   What you can learn:     - Enabel the external interruptsource INT0
*                           - Configurate INT0 for falling edge interrupt
*                           - Write the subroutine for INT0
*
*   Recommended             - High: 0x99
*   Fuse settings:          - Low:  0xE1
*
*   Schematic:       ___________________
*                   |      ATMEGA16     |
*          Vcc ---->|Vcc                |
*          Vcc ---->|RESET              |
*                   |                   |
*          GND ---->|GND                |
*                   |                   |
*            \      |                   |
*    GND ---  \o--->|PD2 (INT0)      PA0|----> DOUT
*                   |                   |
*                   |                   |
*                   |                   |
*
*******************************************************************************/


#include <avr/io.h>
#include <avr/interrupt.h>

int main(void)
{

    DDRA = 0x01;                       // Setup PA1 as output
    PORTD = 0x04;                      // Activate pullupressistor of PD2

    GICR |= (1<<INT0);                 // Enable INT0
    MCUCR |= (1<<ISC01);               // INT0 is executed on falling edge
    sei();                             // Set the I-bit in SREG

  
    for(;;);                           // The main loop stays empty and 
                                       // could contain code you want.

    return 0;                          // This line will never be executed

}

// Interrupt subroutine for external interrupt 0
ISR(INT0_vect)							
{
    PORTA ^= 0x01;                     // Toggle PA0

}
